@@ -1,22 +1,23 @@
 import gobject
-import glib
 import gtk
-import gnome.gnomevfs as gnomevfs
 import hildondesktop
 import hildon
 import dbus
 import gnome.gconf as gconf
-#import diggapi
 import pango
 import cairo
 import xml.dom.minidom as xmlparser
+import urllib2 
 import urllib
-import gio
 PATH = "/apps/digghomewidget"
 CATGR = "/apps/digghomewidget/selected_category_int"
 NUM = "/apps/digghomewidget/number_of_stories"
 THMB = "/apps/digghomewidget/load_thumbnails"
 FIRST = "/apps/digghomewidget/first_run"
+
+gobject.threads_init()
+#gtk.gdk.threads_init()
+
 class diggItem:
     def __init__(self, title, href, topic, diggs):
         self.title = title
@@ -25,89 +26,51 @@ class diggItem:
         self.diggs = diggs
         self.thumbnail = None
 
-class diggLoader:
+
+class diggLoader(gobject.GObject):
     def __init__(self, callback):
+        self.__gobject_init__()
         self.callback = callback
         self.stream = ""
         self.load_thumbnails = True
-        self.total = 0
-        self.fetched = 0
-        self.result = None
-        self.downresult = None
-        self.inputstream = None
+        self.running = False
+        self.list = None
 
-    def stop(self):
-        self.stream = ""
-        self.total = 0
-        self.fetched = 0
-#        if self.result != None:
-#            self.result.complete_in_idle()
-#        if self.downresult != None:
-            #self.downresult.complete_in_idle()
-#            self.inputstream.close()
-#        self.loop.quit()
+    def read_and_return(self, data):#handle, data, result, read_bytes):
+        doc = xmlparser.parseString(data)
+        if doc != None:
+            stories = doc.getElementsByTagName("story")
+            if len(stories) == 0:
+                return None
+            list = []
+            for story in stories:
+                title = story.getElementsByTagName("title")[0]
+                href = story.getElementsByTagName("shorturl")[0]
+                title = title.childNodes[0].data
+                href = href.getAttribute("short_url")
+                diggs = story.getAttribute("diggs")
+                topic = story.getElementsByTagName("topic")[0]
+                topic = topic.getAttribute("name")
+                item = diggItem(title, href, topic, diggs) 
+                if self.load_thumbnails:
+                    thumbnail = None
+                    thumbnail = story.getElementsByTagName("thumbnail")
+                    if thumbnail.length > 0:
+                        thumbnail = thumbnail[0]
+                        thumbnail = thumbnail.getAttribute("src")
+                    try:
+                        if thumbnail != None and thumbnail != "":
+                            g = urllib.urlopen(thumbnail)
+                            img = g.read()
+                            item.thumbnail = img
+                        #print item.thumbnail
+                    except:
+                        item.thumbnail = None
+                list.append(item)
+            self.list = list
+            self.emit("data_loaded", "testi")
 
-    def read_and_return(self, filestream, result):
-        print "Read:"
-        print dir(result)
-        self.result = result
-        temp = filestream.read_finish(result)
-        if temp:
-            self.stream += temp
-            self.fetched += len(temp)
-            filestream.read_async(4096, self.read_and_return)
-        else:
-            doc = xmlparser.parseString(self.stream)
-            if doc != None:
-                stories = doc.getElementsByTagName("story")
-                if len(stories) == 0:
-                    return None
-                list = []
-                for story in stories:
-                    title = story.getElementsByTagName("title")[0]
-                    href = story.getElementsByTagName("shorturl")[0]
-                    title = title.childNodes[0].data
-                    href = href.getAttribute("short_url")
-                    diggs = story.getAttribute("diggs")
-                    topic = story.getElementsByTagName("topic")[0]
-                    topic = topic.getAttribute("name")
-                    item = diggItem(title, href, topic, diggs) 
-            #        print thumbnail
-                    if self.load_thumbnails:
-                        thumbnail = None
-                        thumbnail = story.getElementsByTagName("thumbnail")
-                        if thumbnail.length > 0:
-                            thumbnail = thumbnail[0]
-                            thumbnail = thumbnail.getAttribute("src")
-                        try:
-                            if thumbnail != None and thumbnail != "":
-                                g = urllib.urlopen(thumbnail)
-                                img = g.read()
-                                item.thumbnail = img
-                            #print item.thumbnail
-                        except:
-                            item.thumbnail = None
-                    list.append(item)
-                self.callback(list)
-#            result.complete_in_idle()
-            self.stop()
-
-    def parse_and_return(self, daemonfile, result):
-        print "Parse:"
-        print dir(result)
-        try:
-            self.inputstream = daemonfile.read_finish(result)
-        except gio.Error, e:
-            self.stop()
-            return
-        info = daemonfile.query_info(gio.FILE_ATTRIBUTE_STANDARD_SIZE)
-        self.total = info.get_attribute_uint64(gio.FILE_ATTRIBUTE_STANDARD_SIZE)
-        self.inputstream.read_async(4096, self.read_and_return)
-        self.downresult = result
-
-
-
-    def get_toplist(self, container="", topic="", count=3, hotness="top"):
+    def get_toplist(self, container="", topic="", count=2, hotness="top"):
         selected = ""
         if topic != "":
             selected = "topic/" + topic + "/"
@@ -116,12 +79,17 @@ class diggLoader:
             selected = "container/" + container + "/"
         params = urllib.urlencode({"count":count, "appkey":"http://maemo.org/packages/view/digghomewidget/"})
         try:
-            f = gio.File("http://services.digg.com/stories/"+selected+hotness+"?%s" % params)
-            inputstream = f.read()
-            inputstream.read_async(4096, self.read_and_return)
+            url = "http://services.digg.com/stories/"+selected+hotness+"?%s" % params
+            f = urllib2.urlopen(url)
+            data = f.read()
+            self.read_and_return(data)
         except Exception, e:
             print "Can't connect to Digg.com", e
 
+gobject.type_register(diggLoader)
+gobject.signal_new("data_loaded", diggLoader, gobject.SIGNAL_RUN_FIRST,
+                    gobject.TYPE_NONE, (str,))
+
 class DiggItemWidget(hildon.Button):
     def __init__(self, item):
 #        print inspect.getsourcelines(hildon.Button)
@@ -365,7 +333,8 @@ class DiggHomePlugin(hildondesktop.HomePluginItem):
         self.add(container)
         self.connect("notify::is-on-current-desktop", self.on_desktop)
         self.set_size_request(300, 150)
-        self.loader.get_toplist(container=self.categories[self.category],count=self.count)
+        self.loader.connect("data_loaded", self.update)
+        gobject.idle_add(self.loader.get_toplist, self.categories[self.category], "", self.count)
 #        gtk.Settings.gtk-enable-animations = False
 
     def on_desktop(self, widget, data):
@@ -375,7 +344,8 @@ class DiggHomePlugin(hildondesktop.HomePluginItem):
     def update_button(self, widget):
         self.row.title = "Updating..."
         self.row.queue_draw()
-        self.loader.get_toplist(container=self.categories[self.category],count=self.count)
+        gobject.idle_add(self.loader.get_toplist, self.categories[self.category], "", self.count)
+        #self.loader.get_toplist(container=self.categories[self.category],count=self.count)
 
     def change_topic(self, widget):
         dialog = gtk.Dialog()
@@ -418,8 +388,8 @@ class DiggHomePlugin(hildondesktop.HomePluginItem):
         self.pickerValue = button.get_value()
         
 
-    def update(self, list):
-#        list = diggapi.get_toplist(container=self.category,count=self.count)
+    def update(self, sender, success):
+        list = sender.list
         if list != None:
             children = self.vbox.get_children()
             for child in children:
@@ -428,18 +398,17 @@ class DiggHomePlugin(hildondesktop.HomePluginItem):
                 for item in list:
                     button = DiggItemWidget(item)
                     self.vbox.pack_start(button, False)
-            self.show_all()
+                    button.show()
+            self.queue_draw()
             self.client.set_string(CATGR, self.categories[self.category])
             title = self.categories[self.category]
             title = title.replace("_", "&")
             if title == "":
                 title = "All"
             self.row.title = title 
-            self.row.queue_draw()
         else:
             hildon.hildon_banner_show_information(self, "None", "Can't connect to digg.com. Please check your internet connection.")
 
-
     def selection_changed(self, selector, user_data):
         current_selection = selector.get_current_text()
 #        print current_selection
@@ -490,3 +459,9 @@ class DiggHomePlugin(hildondesktop.HomePluginItem):
 
 hd_plugin_type = DiggHomePlugin
 
+if __name__ == "__main__":
+    import gobject
+    gobject.type_register(hd_plugin_type)
+    obj = gobject.new(hd_plugin_type, plugin_id="plugin_id")
+    obj.show_all()
+    gtk.main()
